package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.entity.Sequence;
import com.qmth.sop.business.mapper.SequenceMapper;
import com.qmth.sop.business.service.SequenceService;
import com.qmth.sop.business.service.TGErrorService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.ErrorEnum;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.lock.MemoryLock;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * <p>
 * 序列表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-14
 */
@Service
public class SequenceServiceImpl extends ServiceImpl<SequenceMapper, Sequence> implements SequenceService {

    @Resource
    MemoryLock memoryLock;

    @Resource
    TGErrorService tgErrorService;

    /**
     * 获取序列号
     *
     * @param name
     * @return
     */
    @Override
    public Long selectNextVal(String name) throws InterruptedException {
        boolean lock = false;
        for (int i = 0; i < SystemConstant.MAX_SEQUENCE_CIRCLE_COUNT; i++) {
            lock = memoryLock.lock(SystemConstant.LOCK_SEQUENCE_PREFIX + name, name, SystemConstant.LOCK_SEQUENCE_TIME_OUT);
            if (lock) {
                try {
                    return this.baseMapper.selectNextVal(name);
                } finally {
                    memoryLock.unlock(SystemConstant.LOCK_SEQUENCE_PREFIX + name);
                }
            } else {
                Thread.sleep(SystemConstant.SEQUENCE_THREAD_SLEEP);
            }
        }
        if (!lock) {
            tgErrorService.saveExamTgError(ErrorEnum.SEQUENCE.name(), ErrorEnum.SEQUENCE.getTitle());
            throw ExceptionResultEnum.ERROR.exception("获取序列失败");
        }
        return null;
    }
}
